;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;===============================
;ZX81 versions of the Applesoft commands...
;
;  HLIN x1,x2 at y 
;    Draws a horizontal line from x1,y to x2,y
;
;  VLIN y1,y2 at x 
;    Draws a vertical line from x,y1 to x,y2
;===============================
ORG 4084

@PrintAt=08F5			;PRINT AT subroutine
@Scanning=0F55			;The expression evaluator
@Print1=0AD5			;PRINT-1 subroutine
@FPtoA=15CD			;Floating-point to A subroutine
@Errsp=4002			;Address of 1st item on machine stack
@DFile=400C			;Start of DFile
@DFxCC=400E			;Address of PRINT position in display file
@Vars=4010			;Start of VARS
@CHAdd=4016			;Address of next chr to be interpreted
;================================
@JumpTable
	JR Rite			;16516
	JR Left			;16518
	JR Down			;16520
	JR Up			;16522
	JP Box			;16524
	JP PokeDF		;16527
;================================
;Set Reg A to 8bit parameter value
;================================
@SetA
	LD (CHAdd),HL		;Start of parameter
	CALL Scanning		;Call the expression evaluator
	PUSH HL 		;PUSH BASIC line addr
	CALL FPtoA		;A=Results
	POP HL 			;POP BASIC line addr
	RET
;================================
; Get the line length and CODE X$
; parameters
;================================
@GetPars
	LD HL,(CHAdd)		;HL=CHAdd
	INC HL			;Line length address
	CALL SetA		;Get Length
	AND A			;Length=0
	JR Z ErrorB		;Yes, error B
	LD (Count+1),A		;Set length counter
	INC HL			;CODE X$ address
	CALL SetA		;Get CODE X$
@Count
	LD B,00			;B=length
	RET			;A=CODE X$, B=Length
@ErrorB
	RST 08			;Error restart
	0A			;Error Report B: Integer out of range
;================================
; Check for EOL
;================================
@LineEnd
	POP HL			;POP CHAdd
	LD (CHAdd),HL		;Set CHAdd
	LD A,(HL)		;Test for EOL marker
	CP 76			;EOL?
	JR Z SPRet		;Yes, return via ROMs "NEXT-LINE" routine
	CALL Print1		;Else start next PRINT statement
	RET
@SPRet
	LD SP,(Errsp)		;Set stack pointer to ROMs "NEXT-LINE" routine
	RET			;Return via "NEXT-LINE" routine
;================================
;Rite, draw horizontal line left to right
;
; USAGE:
;
;  PRINT AT Y,X;USR Rite;W,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   W=Width, 1 to 255,  0=Error B
;   C$=Chr to print
;================================
@Rite
	CALL GetPars		;Get line pars, A=CODE X$ B=Length
	PUSH HL			;PUSH CHAdd
@PrintR
	RST 10			;Print A (VARS overflow causes Error 5)
	DJNZ PrintR		;Loop until B=0
	JR LineEnd
;================================
;Left, draw horizontal line right to left
;
; USAGE:
;
;  PRINT AT Y,X;USR Left;W,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   W=Width, 1 to 255,  0=Error B
;   C$=Chr to print
;================================
@Left
	CALL GetPars		;Get line pars, A=CODE X$ B=Length
	PUSH HL			;PUSH CHAdd
	RST 10			;Print A
	LD HL,(DFxCC)		;Get DFxCC
	DEC HL			;Move print position back by 2
	DEC HL
	BIT 6,(HL)		;EOL?
	JR Z +1			;No, move on
	DEC HL			;Backup one more
	LD (DFxCC),HL		;Set (DFxCC)=next print position
	EX DE,HL		;Test for DFile underflow
	LD HL,(DFile)		;Get DFile
	SBC HL,DE		;CHAdd<DFile?
	JR NC LineEnd		;Yes, underflow condition, stop printing
	DJNZ Left+4		;Loop until B=0
	JR LineEnd		
;================================
;Down/Up, draw vertical lines
;
; USAGE:
;
;  PRINT AT Y,X;USR Down;H,CODE C$
;  PRINT AT Y,X;USR Up;H,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   H=Height, 1 to 24, 0=Error B
;   C$=Chr to print
;================================
@Down
	LD A,20			;Offset value
	LD HL,0019		;Add instruction
	JR +5
@Up
	LD A,22			;Offset value
	LD HL,52ED		;Sub instruction
	LD (SetV+1),A		;Set offset value
	LD (AddSub),HL		;Set Add/Sub instruction
	CALL GetPars		;Get line pars, A=CODE X$ B=Length
	PUSH HL			;PUSH CHAdd
@SetV
	LD DE,0000		;Offset to next line
	RST 10			;Print A
	LD HL,(DFxCC)		;Get DFxCC
	AND A			;Clear CY
@AddSub
	00 00			;Add or Subtract line offset
	LD (DFxCC),HL		;Set DFxCC
	EX DE,HL
	LD HL,(DFile)		;Upper DFile overflow
	SBC HL,DE
	JR NC LineEnd		;If overflow stop printing
	LD HL,(Vars)		;Lower DFile overflow
	SBC HL,DE
	JR C LineEnd		;If overflow stop printing
	DJNZ SetV		;Loop until B=0
	JR LineEnd			
;================================
;Box, draw solid rectangle
;
; USAGE:
;
;  PRINT AT Y,X;USR Box;H,W,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   H=Height, 1 to 24, 0=Error B
;   W=Width
;   C$=Fill Chr
;================================
@Box
	CALL GetPars		;Get line pars, A=C=Width  B=height
	AND A			;Width=0?
	JP Z ErrorB		;Yes, error B
	LD E,(IY+57)		;Get SPosnX
	DEC E			;E=maximum allowable length
	SUB E			;Width>Max?
	JR C +1			;No, move on
	LD C,E			;Set width to max
	LD A,C
	LD (Width+1),A		;Set Width
	LD A,B			;A=Height
	LD (Width-1),A		;Set Height
	LD A,21			;33 bytes/row
	SUB C			;C=Width
	LD (Offset+1),A		;Set offset, offset=33-width
	INC HL			;Fill chr addr
	CALL SetA		;Get fill chr
	PUSH HL			;PUSH CHAdd
	LD C,00			;C=Height
@Width
	LD B,00			;Loop counter
	RST 10			;Print A
	DJNZ Width+2		;Print A until B=0
	DEC C			;Decrease height counter
	JP Z LineEnd		;When C=0 check for EOL
	LD HL,(DFxCC)		;Get address of PRINT position
@Offset
	LD DE,0000
	ADD HL,DE		;Add offset
	EX DE,HL		;DE=DFxCC
	LD HL,(Vars)		;Check for DFile overflow
	SBC HL,DE		;Test overflow condition
	EX DE,HL		;HL=DFxCC
	JP C LineEnd		;If overflow condition stop printing
	LD (DFxCC),HL		;Set PRINT position
	JR Width		;Loop until C=0
;================================
; USAGE:
;
;  PRINT USR PokeDF;Y,X,CODE C$
;
;   Y=Ycoord
;   X=Xcoord
;   C$=Chr to POKE
;================================
@PokeDF
	LD HL,(CHAdd)		;HL=CHAdd
	INC HL			;Ycoord addr
	CALL SetA		;Get Ycoord
	LD (RowCount+1),A	;Set row counter
	INC HL			;Xcoord addr
	PUSH HL			;PUSH CHadd
	LD HL,(DFile)		;Get DFile+1
	INC HL
@RowCount
	LD A,00			;Add A rows to DFile
	AND A			;Rows=0?
	JR Z Rows0		;Yes, move on
	LD B,A			;B=row counter
	LD DE,0021		;33 bytes/row
@AddRow
	ADD HL,DE
	DJNZ AddRow
@Rows0
	LD (RowBytes+1),HL	;Set DFile Ycoord offset
	POP HL			;POP CHadd
	CALL SetA		;Get Xcoord
	LD (RowBytes+4),A	;Set Xcoord
	INC HL			;POKE value addr
	CALL SetA		;Get POKE value
	PUSH HL			;PUSH CHadd
@RowBytes
	LD HL,0000		;HL=DFile+33*Ycoord
	LD BC,0000		;BC=Xcoord
	ADD HL,BC		;HL=DFile+33*Ycoord+Xcoord
	LD (HL),A		;POKE DFile
	POP HL			;POP CHadd
	LD A,(HL)		;Test for EOL marker
	CP 76			;EOL?
	JP Z SPRet		;Yes, return via ROMs "NEXT-LINE" routine
	JR PokeDF+3		;Next POKE set
	00